@@ -16,13 +16,32 @@
 
 #include <numeric>
 
+#include "frontier.h"
 #include "path.h"
 
+IndexT NodePath::noSplit = 0;
+
+
+void NodePath::setNoSplit(IndexT bagCount) {
+  noSplit = bagCount;
+}
+
 
 IdxPath::IdxPath(IndexT idxLive_) :
   idxLive(idxLive_),
-  relFront(vector<unsigned int>(idxLive)),
-  pathFront(vector<unsigned char>(idxLive)),
-  offFront(vector<uint_least16_t>(idxLive)) {
-  iota(relFront.begin(), relFront.end(), 0);
+  smIdx(vector<IndexT>(idxLive)),
+  pathFront(vector<PathT>(idxLive)) {
+  iota(smIdx.begin(), smIdx.end(), 0);
 }
+
+
+/**
+     @brief Sets to non-extinct path coordinates.
+   */
+void NodePath::init(const Frontier* frontier, const IndexSet& iSet) {
+  frontIdx = iSet.getSplitIdx();
+  bufRange = iSet.getBufRange();
+  idxStart = frontier->idxStartUpcoming(iSet);
+}
+  
+
