@@ -16,18 +16,20 @@
 #ifndef CORE_SUMCOUNT_H
 #define CORE_SUMCOUNT_H
 
+#include "typeparam.h"
+
+
 #include <vector>
 using namespace std;
 
 /**
    @brief Row sum / count record for categorical indices.
  */
-class SumCount {
+struct SumCount {
   double sum;
-  unsigned int sCount;
-
- public:
+  IndexT sCount;
 
+  
   SumCount(double sum_,
            unsigned int sCount_) : sum(sum_), sCount(sCount_) {
   }
@@ -45,6 +47,14 @@ class SumCount {
     return sCount;
   }
 
+
+  /**
+     @return sample mean.
+   */
+  inline double mean() const {
+    return sum / sCount;
+  }
+
   
   /**
      @brief Determines whether a node is splitable and accesses sum field.
